"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var inferno_1 = require("inferno");
var inferno_shared_1 = require("inferno-shared");
var utils_1 = require("./utils");
function renderStyleToString(style) {
    if (inferno_shared_1.isStringOrNumber(style)) {
        return style;
    }
    else {
        var styles = [];
        for (var styleName in style) {
            var value = style[styleName];
            var px = inferno_shared_1.isNumber(value) && !inferno_1.internal_isUnitlessNumber.has(styleName) ? 'px' : '';
            if (!inferno_shared_1.isNullOrUndef(value)) {
                styles.push(utils_1.toHyphenCase(styleName) + ":" + utils_1.escapeText(value) + px + ";");
            }
        }
        return styles.join();
    }
}
exports.renderStyleToString = renderStyleToString;
function renderAttributes(props) {
    var outputAttrs = [];
    var propsKeys = (props && Object.keys(props)) || [];
    propsKeys.forEach(function (propKey, i) {
        var value = props[propKey];
        switch (propKey) {
            case 'children':
            case 'dangerouslySetInnerHTML':
            case 'style':
                return;
            default:
                if (inferno_shared_1.isStringOrNumber(value)) {
                    outputAttrs.push(utils_1.escapeText(propKey) + '="' + utils_1.escapeText(value) + '"');
                }
                else if (inferno_shared_1.isTrue(value)) {
                    outputAttrs.push(utils_1.escapeText(propKey));
                }
        }
    });
    return outputAttrs;
}
exports.renderAttributes = renderAttributes;
